<?php

namespace FiloBlu\Flow\Model\Channel\In;

class Anagrafica extends AbstractModel {

    /**
     * @var Monolog\Logger
     */
    protected $_logger;
    protected $_storesByLocale = array();
    protected $_channel_config;
    protected $_resourceconnection;
    protected $_helper;

    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel\Config $channelConfig,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \FiloBlu\Flow\Helper\Data $helper
    ) {
        parent::__construct($loggerProvider, $channelConfig);
        $this->_logger = $loggerProvider->getLogger();
        $this->_channel_config = $channelConfig;
        $this->_resourceconnection = $resourceConnection;
        $this->_helper = $helper;

        $this->_storeByLocale = $helper->getAllStoresCodeByLocale();

    }

    public function load($channelname) {
        $this->init($channelname);
        return $this;
    }

    public function getSimplefields($data, $meta_file)
    {

        foreach ($data as $field => $value) {
            $fieldMap = $this->map->get($field);
            $value = addslashes($value);

            if (!isset($fieldMap["skip"]) && (trim($value) != '' || isset($fieldMap["allowNull"]))) {
                $to_field = "";
                if (isset($fieldMap["to"]) && $fieldMap["to"]) {
                    $to_field = $fieldMap["to"];
                    if (!isset($fieldMap['actions']) || !in_array('_translate_attribute', $fieldMap['actions'])) {
                        //devo inserire una riga per ogni attributo
                        $template_bind = array(
                            'e' => $data['sku'],
                            'a' => $to_field,
                            'v' => trim($value),
                            'meta_file' => $meta_file,
                            's' => '',
                            'meta_ref_id' => 0,
                            'meta_processed' => 0,
                            'meta_insert_time' => date("Y-m-d H:i:s", time())
                        );

                        try {

                            $storeViews = array('s'=>'');

                            if (isset($fieldMap["stores"]) && count($fieldMap["stores"]) > 0 && $fieldMap["stores"]) {
                                $storeViews = $fieldMap["stores"];
                            } else if (isset($fieldMap["locale"]) && count($fieldMap["locale"]) > 0 && $fieldMap["locale"]) {
                                $storeViews = $this->_storeByLocale[$fieldMap["locale"]];
                            }

                            foreach ($storeViews as $store) {
                                $template_bind['s'] = $store;
                                $binds["simple"][] = $template_bind;
                            }

                        } catch (\Exception $e) {
                            // $writeConnection->rollback();
                            throw new \Exception($e->getMessage());
                        }
                    }
                }

            }
            //le azioni le conservo per inserirle alla fine
            if (isset($fieldMap["actions"])) {
                foreach ($fieldMap["actions"] as $action) {
                    $a = array("value" => $value, "stores" => array());
                    if (isset($to_field) && ($action == '_translate_attribute' || $action == '_set_default_locale')) {
                        $a['e'] = $to_field;
                    }
                    if (isset($fieldMap["stores"]) && count($fieldMap["stores"]) > 0 && $fieldMap["stores"]) {
                        $a['stores'] = $fieldMap["stores"];
                    } else if (isset($fieldMap["locale"]) && count($fieldMap["locale"]) > 0 && $fieldMap["locale"]) {
                        $storeViews = $this->_storeByLocale($fieldMap["locale"]);
                        foreach ($storeViews as $store) {
                            $a['stores'][] = $store;
                        }
                    }


                    $binds["actions"][$action][] = $a;
                }
            }
        }

        return $binds;

    }

    public function getActionfields($data,$actions,$meta_file) {


        foreach($actions as $a => $values){
            foreach($values as $v) {
                if(in_array($a, array('_add_related', '_add_upsell', '_add_crosssell'))){
                    //$padre = explode('#',$data['cod_configurabile']);
                    $bind = array(
                        'e' => $data['sku'],
                        'a' => $a,
                        'v' => $v['value'],
                        'meta_file' => $meta_file,
                        's' => '',
                        'meta_ref_id' => (isset($v['e']) ? 1 : 2),
                        'meta_processed' => 0,
                        'meta_insert_time' => date("Y-m-d H:i:s", time())
                    );
                }
                else{
                    $bind = array(
                        'e' => (isset($v['e']) ? $v['e'] : $data['sku']),
                        'a' => $a,
                        'v' => $v['value'],
                        'meta_file' => $meta_file,
                        's' => '',
                        'meta_ref_id' => (isset($v['e']) ? 1 : 2),
                        'meta_processed' => 0,
                        'meta_insert_time' => date("Y-m-d H:i:s", time())
                    );
                }
                if (count($v['stores']) > 0) {
                    $storeViews = $v['stores'];
                    foreach ($storeViews as $store) {
                        $bind['s'] = $store;
                        $actions_parserd[] = $bind;
                    }
                } else {
                    $actions_parserd[] = $bind;
                }
            }
        }


        return $actions_parserd;

    }

    public function insertData($data){

        $flow = $this->getFile()->getData('flow');
        $model =  \Magento\Framework\App\ObjectManager::getInstance()->create('\FiloBlu\Flow\Model' . "\\" . $flow);

        if (!$model)
            return false;

        $resource = $this->_resourceconnection;

        $writeConnection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $tableName = $resource->getTableName('flow_from_eavs');

        $meta_file = $this->getFile()->getId();
        $actions = array();

        $query = "INSERT INTO {$tableName} (e,a,v,s,meta_file,meta_ref_id,meta_processed,meta_insert_time) VALUES (:e,:a,:v,:s,:meta_file,:meta_ref_id,:meta_processed,:meta_insert_time)";

        $binds_raw = $this->getSimplefields($data,$meta_file);
        $actions_raw =$this->getActionfields($data,$binds_raw["actions"],$meta_file);

        $binds = array_merge($binds_raw["simple"],$actions_raw);

        try {

            $writeConnection->insertMultiple($tableName, $binds);

        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }


    }

    public function receiveFile($file){

        //$this->_logger->info(__METHOD__);

        if (!$this->_connector)
            throw new \Exception('no connector joined to channel');

        $file_channel_config = \Magento\Framework\App\ObjectManager::getInstance()->create('\FiloBlu\Flow\Model\Channel\Config')->load($file->getChannel())->getConfigData();

        // se il flag è true, quando viene caricato un file anagrafica, automaticamente viene duplicato per il flusso ite
        // passo anche il nome del channel ite con la configurazione corretta - se il campo è vuoto, il default sarà "ite"
        $ite = $file_channel_config['config']['copy_to_ite'];
        $ite_channel_name = ($file_channel_config['config']['ite_channel_name']) ? $file_channel_config['config']['ite_channel_name'] : 'ite';

        if($ite){
            $received = $this->_connector->receiveResource($file->getName(), 'ite_'.date('Ymd_Hmi'));
        } else {
            $received = $this->_connector->receiveResource($file->getName());
        }
        //Se ho copiato il fine creo una riga per il flusso
        if(is_array($received)){

            $copy = $received['copy'];
            $received = $received['basefile'];
            $model = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Inboundflow');;
            $model->setName($copy);
            $model->setFlow('From\Product');
            $model->setType('file');
            $model->setChannel($ite_channel_name);
            $model->setRetry(0);
            $model->setStatus('received');
            $model->setPriority('normal');
            try{
                $model->save();
            }catch (\Exception $e){
                $this->_logger->info($e->getMessage());
            }
        }
        elseif ($received === false) {
            throw new \Exception('no remote files found');
        }
        return $received;

    }


    protected function checkDuplicates($tableName,$bind){

        // $this->_logger->info(__METHOD__);

        $resource = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Framework\App\ResourceConnection');
        $readConnection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);

        $query = "SELECT count(*) FROM {$tableName} WHERE e = '{$bind["e"]}' AND a = '{$bind["a"]}' AND v = '{$bind["v"]}' AND meta_file = '{$bind["meta_file"]}' AND s = '{$bind["s"]}'";
        $result = $readConnection->fetchOne($query);
        if($result > 0){
            return true;
        }
        return false;
    }


}
